@@ -1,6 +1,5 @@
 # This should be equivalent to running regrtest.py from the cmdline.
 # It can be especially handy if you're in an interactive shell, e.g.,
 # from test import autotest.
-
-import regrtest
-regrtest.main()
+from test.libregrtest.main import main
+main()
